/****************************************************************************

    PROGRAM: esound.c

    PURPOSE: sound special effects

    FUNCTIONS:

	WinMain() - calls initialization function, processes message loop
	InitApplication() - initializes window data and registers window
	InitInstance() - saves instance handle and creates main window
	MainWndProc() - processes messages
	About() - processes messages for "About" dialog box

    COMMENTS:

	Windows can have several copies of your application running at the
	same time.  The variable hInst keeps track of which instance this
	application is so that processing will be to the correct window.

****************************************************************************/

#include <windows.h>                /* required for all Windows applications */
#include <windowsx.h>                /* neat capabilities */
#include <mmsystem.h>                /* multimedia extensions */
#include <commdlg.h>
#include <stdlib.h>
#include <memory.h>
#include "esound.h"                /* specific to this program              */

HANDLE hInst;                       /* current instance                      */
HWND hwndMain;                  //handle to main window
int nAttenuation, nAttenuation2;				//attenuation
WORD nDelay, nDelay2;
int nGain;						//gain on initial signal

#define DELAYRANGE					1000
#define ATTENUATIONRANGE			100
#define GAINRANGE					100


/****************************************************************************

    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop

    COMMENTS:

	Windows recognizes this function by name as the initial entry point
	for the program.  This function calls the application initialization
	routine, if no other instance of the program is running, and always
	calls the instance initialization routine.  It then executes a message
	retrieval and dispatch loop that is the top-level control structure
	for the remainder of execution.  The loop is terminated when a WM_QUIT
	message is received, at which time this function exits the application
	instance by returning the value passed by PostQuitMessage().

	If this function must abort before entering the message loop, it
	returns the conventional value NULL.

****************************************************************************/

#ifdef __BORLANDC__
    #pragma argsused
#endif
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow)
{
    MSG msg;                                 /* message                      */

    if (!hPrevInstance)                  /* Other instances of app running? */
	if (!InitApplication(hInstance)) /* Initialize shared things */
	    return (FALSE);              /* Exits if unable to initialize     */

    /* Perform initializations that apply to a specific instance */

    if (!InitInstance(hInstance, nCmdShow))
	return (FALSE);

    /* Acquire and dispatch messages until a WM_QUIT message is received. */

    while (GetMessage(&msg,        /* message structure                      */
	    NULL,                  /* handle of window receiving the message */
	    NULL,                  /* lowest message to examine              */
	    NULL))                 /* highest message to examine             */
	{
	TranslateMessage(&msg);    /* Translates virtual key codes           */
	DispatchMessage(&msg);     /* Dispatches message to window           */
    }
    return (msg.wParam);           /* Returns the value from PostQuitMessage */
}
/****************************************************************************

    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class

    COMMENTS:

	This function is called at initialization time only if no other
	instances of the application are running.  This function performs
	initialization tasks that can be done once for any number of running
	instances.

	In this case, we initialize a window class by filling out a data
	structure of type WNDCLASS and calling the Windows RegisterClass()
	function.  Since all instances of this application use the same window
	class, we only need to do this when the first instance is initialized.


****************************************************************************/
BOOL InitApplication(HINSTANCE hInstance)
{
    WNDCLASS  wc;

    /* Fill in window class structure with parameters that describe the       */
    /* main window.                                                           */

    wc.style = NULL;                    /* Class style(s).                    */
    wc.lpfnWndProc = (WNDPROC)MainWndProc;       /* Function to retrieve messages for  */
					/* windows of this class.             */
    wc.cbClsExtra = 0;                  /* No per-class extra data.           */
    wc.cbWndExtra = 0;                  /* No per-window extra data.          */
    wc.hInstance = hInstance;           /* Application that owns the class.   */
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(GRAY_BRUSH);
    wc.lpszMenuName =  "esoundMenu";   /* Name of menu resource in .RC file. */
    wc.lpszClassName = "esoundWClass"; /* Name used in call to CreateWindow. */

    /* Register the window class and return success/failure code. */

    return (RegisterClass(&wc));

}
/****************************************************************************

    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window

    COMMENTS:

	This function is called at initialization time for every instance of
	this application.  This function performs initialization tasks that
	cannot be shared by multiple instances.

	In this case, we save the instance handle in a static variable and
	create and display the main program window.

****************************************************************************/

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
    HWND            hWnd;               /* Main window handle.                */

    /* Save the instance handle in static variable, which will be used in  */
    /* many subsequence calls from this application to Windows.            */

    hInst = hInstance;

    /* Create a main window for this application instance.  */

    hWnd = hwndMain = CreateWindow(
	"esoundWClass",                /* See RegisterClass() call.          */
	"Effects Sample Application",   /* Text for window title bar.         */
	WS_OVERLAPPEDWINDOW,            /* Window style.                      */
	CW_USEDEFAULT,                  /* Default horizontal position.       */
	CW_USEDEFAULT,                  /* Default vertical position.         */
	CW_USEDEFAULT,                  /* Default width.                     */
	CW_USEDEFAULT,                  /* Default height.                    */
	NULL,                           /* Overlapped windows have no parent. */
	NULL,                           /* Use the window class menu.         */
	hInstance,                      /* This instance owns this window.    */
	NULL                            /* Pointer not needed.                */
    );

    /* If window could not be created, return "failure" */

    if (!hWnd)
	return (FALSE);

    /* Make the window visible; update its client area; and return "success" */

    ShowWindow(hWnd, nCmdShow);  /* Show the window                        */
    UpdateWindow(hWnd);          /* Sends WM_PAINT message                 */
    return (TRUE);               /* Returns the value from PostQuitMessage */

}
/****************************************************************************

    FUNCTION: MainWndProc(HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages

    MESSAGES:

	WM_COMMAND    - application menu (About dialog box)
	WM_DESTROY    - destroy window

    COMMENTS:

	To process the IDM_ABOUT message, call MakeProcInstance() to get the
	current instance address of the About() function.  Then call Dialog
	box which will create the box according to the information in your
	esound.rc file and turn control over to the About() function.  When
	it returns, free the intance address.

****************************************************************************/

LPWAVEHDR lp_wavehdr;
PCMWAVEFORMAT FAR *lp_waveformat;
int _huge *hp_wavedata;
HWAVEOUT h_wave;
BOOL b_waveopen = FALSE;


LRESULT CALLBACK __export MainWndProc(HWND hWnd, UINT message, WPARAM wParam,
			    LPARAM lParam)
{
PAINTSTRUCT             ps;
char                    szFileName[_MAX_PATH];
char szFilter[] = "WAV Files (*.WAV)\0*.WAV\0\0";
int FAR *lpFIFO, FAR * lpFIFO2;
DWORD nIndex, nIndex2, i;
LONG lOutput, lReflection, lReflection2;
DWORD dwFIFOSize, dwFIFOSize2;

    switch (message)
    {
		/* the buffer has played out */
		case MM_WOM_DONE:
			cleanUpWave (h_wave, lp_wavehdr, hp_wavedata);
			break;

		case WM_PAINT:
			BeginPaint(hWnd,&ps);
			EndPaint (hWnd, &ps);
			break;
		case WM_COMMAND:
		    switch (wParam) {
				case IDM_ECHO:
				    if (initOpenFile (szFileName, "WAV Files", "wav", szFilter) !=0) {
						MessageBox (hWnd, "Could not get file name.", "", MB_OK);
						break;
					}
					if (loadWaveFile (0, szFileName, &lp_wavehdr, &lp_waveformat, 
										&hp_wavedata, &h_wave, hWnd) !=0) {
						MessageBox (NULL, "Error loading wave file.", "", MB_OK);
						break;
					}

					/* get echo parameters */
				    if (!DialogBox(hInst,                 //current instance
							"ECHOPARAMETERS",          //resource to use
							hWnd,                        //parent handle
							EchoParameters))
						break;
					/* allocate delay buffer */
					/* size in bytes */
					dwFIFOSize = (DWORD)(lp_waveformat->wf.nSamplesPerSec/1000*nDelay)*2;

					/* allocate FIFO */
					lpFIFO = (int FAR *) GlobalAllocPtr(GPTR, (DWORD)dwFIFOSize);
					if (!lpFIFO) {
						cleanUpWave (h_wave, lp_wavehdr, hp_wavedata);
						GlobalFreePtr( lp_waveformat );
						MessageBox (hWnd, "No memory", "", MB_OK);
						break;
					}

					/* add echo */
					for (i=0, nIndex=0; i<lp_wavehdr->dwBufferLength/2; i++) {
						/* initialize the first FIFO full of sound data with silence */
						if (i < dwFIFOSize/2)
							lpFIFO[nIndex] = 0x8000;
						/* compute attenuated reflection */
						lReflection = ((LONG)lpFIFO[nIndex]+0x7fffL)*(LONG)nAttenuation;
						lReflection /= ATTENUATIONRANGE;
						/* compute output */
						lOutput = ((LONG)hp_wavedata[i]+0x7fffL)*nGain;
						lOutput /= GAINRANGE;
						lOutput += lReflection;
						/* save sample for later use as reflection */
						lpFIFO[nIndex] = hp_wavedata[i];
						/* echo is done in place, i.e. current sample is replaced
						with echo sample */
						hp_wavedata[i] = (int)(lOutput-0x7fffL);
						/* update FIFO index */
						nIndex = (nIndex + 1) % (dwFIFOSize/2);
					}

					GlobalFreePtr( lp_waveformat );
					GlobalFreePtr( lpFIFO );
					
					/* play the wave file */
					if (waveOutWrite (h_wave, lp_wavehdr, sizeof (WAVEHDR))) {
						MessageBox (NULL, "Error writing wave message.", "", MB_OK);
						break;
					}
					break;

				case IDM_REVERB:
				    if (initOpenFile (szFileName, "WAV Files", "wav", szFilter) !=0) {
						MessageBox (hWnd, "Could not get file name.", "", MB_OK);
						break;
					}
					if (loadWaveFile (0, szFileName, &lp_wavehdr, &lp_waveformat, 
										&hp_wavedata, &h_wave, hWnd) !=0) {
						MessageBox (NULL, "Error loading wave file.", "", MB_OK);
						break;
					}

					/* get reverb parameters */
				    if (!DialogBox(hInst,                 //current instance
							"REVERBPARAMETERS",          //resource to use
							hWnd,                        //parent handle
							ReverbParameters))
						break;
					/* allocate delay buffers */
					/* size in bytes */
					dwFIFOSize = (DWORD)(lp_waveformat->wf.nSamplesPerSec/1000*nDelay)*2;
					dwFIFOSize2 = (DWORD)(lp_waveformat->wf.nSamplesPerSec/1000*nDelay2)*2;

					/* allocate FIFO */
					lpFIFO = (int FAR *) GlobalAllocPtr(GPTR, (DWORD)dwFIFOSize);
					if (!lpFIFO) {
						cleanUpWave (h_wave, lp_wavehdr, hp_wavedata);
						GlobalFreePtr( lp_waveformat );
						MessageBox (hWnd, "No memory", "", MB_OK);
						break;
					}
					lpFIFO2 = (int FAR *) GlobalAllocPtr(GPTR, (DWORD)dwFIFOSize2);
					if (!lpFIFO2) {
						cleanUpWave (h_wave, lp_wavehdr, hp_wavedata);
						GlobalFreePtr( lp_waveformat );
						GlobalFreePtr( lpFIFO );
						MessageBox (hWnd, "No memory", "", MB_OK);
						break;
					}

					/* add echo and reverb */
					for (i=0, nIndex=0, nIndex2=0; i<lp_wavehdr->dwBufferLength/2; i++) {
						/* initialize the first FIFO full of sound data with silence */
						if (i < dwFIFOSize/2) {
							lpFIFO[nIndex] = 0x8000;
							lpFIFO2[nIndex2] = 0x8000;
						}
						/* compute attenuated reflections */
						lReflection = ((LONG)lpFIFO[nIndex]+0x7fffL)*(LONG)nAttenuation;
						lReflection /= ATTENUATIONRANGE;
						lReflection2 = ((LONG)lpFIFO2[nIndex2]+0x7fffL)*(LONG)nAttenuation2;
						lReflection2 /= ATTENUATIONRANGE;
						/* compute output */
						lOutput = ((LONG)hp_wavedata[i]+0x7fffL)*nGain;
						lOutput /= GAINRANGE;
						lOutput += lReflection;
						lOutput += lReflection2;
						/* save sample for later use as reflection */
						lpFIFO[nIndex] = hp_wavedata[i];
						lpFIFO2[nIndex2] = (int)(lOutput-0x7fffL);
						/* echo and reverb is done in place, i.e. current sample is replaced
						with echo sample */
						hp_wavedata[i] = (int)(lOutput-0x7fffL);
						/* update FIFO indices */
						nIndex = (nIndex + 1) % (dwFIFOSize/2);
						nIndex2 = (nIndex2 + 1) % (dwFIFOSize2/2);
					}

					GlobalFreePtr( lp_waveformat );
					GlobalFreePtr( lpFIFO );
					GlobalFreePtr( lpFIFO2 );
					
					/* play the wave file */
					if (waveOutWrite (h_wave, lp_wavehdr, sizeof (WAVEHDR))) {
						MessageBox (NULL, "Error writing wave message.", "", MB_OK);
						break;
					}
					break;

				default:
				    return (DefWindowProc(hWnd, message, wParam, lParam));
		} //End switch (wParam)
	    break;
		case WM_CREATE:
		    break;
		case WM_DESTROY:
			PostQuitMessage (0);
		    break;
		default:
		    return (DefWindowProc(hWnd, message, wParam, lParam));
    } //End switch (message)
    return (NULL);
} //End function (MainWndProc)
/****************************************************************************
    FUNCTION: initOpenFile
****************************************************************************/
int initOpenFile (char FAR *szFileName, char FAR *szTitle,
		    char FAR *szExt, char FAR *szFilter)
{
OPENFILENAME 	ofn;
DWORD 			dwrc;

	szFileName[0]=0;

	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = NULL;
	ofn.hInstance = NULL;
	ofn.lpstrFilter = szFilter;
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter = 0;
	ofn.nFilterIndex = 0;
	ofn.lpstrFile = szFileName;
	ofn.nMaxFile = _MAX_PATH;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.lpstrTitle = szTitle;
	ofn.Flags = OFN_CREATEPROMPT;
	ofn.nFileOffset = 0;
	ofn.nFileExtension = 0;
	ofn.lpstrDefExt = szExt;
	ofn.lCustData = 0L;
	ofn.lpfnHook = NULL;
	ofn.lpTemplateName = NULL;

	dwrc = 0;
	if (GetOpenFileName (&ofn) == 0) {
		dwrc = CommDlgExtendedError ();
	}
	return (int)dwrc;
}
/****************************************************************************
    FUNCTION: EchoParameters
    PURPOSE:  Processes messages for echo parameters dialog box
****************************************************************************/
BOOL CALLBACK __export EchoParameters(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
WORD nPos;
WORD wRange;
HWND hwndScroll;
static WORD nAttenuationPos = ATTENUATIONRANGE/2;
static WORD nDelayPos = DELAYRANGE/2;
static WORD nGainPos = GAINRANGE/2;
static HWND hwndDelay, hwndGain;
static HWND hwndAttenuation;

    switch (message) {
		case WM_INITDIALOG:                   //message: initialize dialog box
			hwndDelay = GetDlgItem (hDlg, IDC_SCROLLBAR1);
			hwndAttenuation = GetDlgItem (hDlg, IDC_SCROLLBAR2);
			hwndGain = GetDlgItem (hDlg, IDC_SCROLLBAR3);
			SetScrollRange (hwndDelay, SB_CTL, 0, DELAYRANGE, FALSE);
			SetScrollPos (hwndDelay, SB_CTL, nDelayPos, FALSE);
			SetScrollRange (hwndAttenuation, SB_CTL, 0, ATTENUATIONRANGE, FALSE);
			SetScrollPos (hwndAttenuation, SB_CTL, nAttenuationPos, FALSE);
			SetScrollRange (hwndGain, SB_CTL, 0, GAINRANGE, FALSE);
			SetScrollPos (hwndGain, SB_CTL, nGainPos, FALSE);
		    return (TRUE);
		case WM_COMMAND:                      //message: received a command
		    if (wParam == IDOK) {                //"OK" box selected?
				EndDialog(hDlg, TRUE);        //Exits the dialog box
				return (TRUE);
		    } //End if (OK button)
		    else if (wParam == IDCANCEL) {                //"CANCEL" box selected?
				EndDialog(hDlg, FALSE);        //Exits the dialog box
				return (TRUE);
		    } //End if (OK button)
		    break;
		case WM_VSCROLL:
			hwndScroll = (HWND) HIWORD (lParam);
			if (hwndScroll == hwndDelay) {
				wRange = DELAYRANGE;
				nPos = nDelayPos;
			}
			else if (hwndScroll == hwndAttenuation) {
				wRange = ATTENUATIONRANGE;
				nPos = nAttenuationPos;
			}
			else {
				wRange = GAINRANGE;
				nPos = nGainPos;
			}
			switch (wParam) {
				case SB_PAGEDOWN:
						nPos += wRange/10;
				case SB_LINEDOWN:
						nPos = min (wRange, nPos+10);
						break;
				case SB_PAGEUP:
						nPos -= wRange/10;
				case SB_LINEUP:
						nPos = max (0, nPos-10);
						break;
				case SB_TOP:
						nPos = 0;
						break;
				case SB_BOTTOM:
						nPos = wRange;
						break;
				case SB_THUMBPOSITION:
				case SB_THUMBTRACK:
						nPos = LOWORD (lParam);
						break;
				default:
					break;
			} /* end switch (wParam) */

			SetScrollPos (hwndScroll, SB_CTL, nPos, TRUE);
			if (hwndScroll == hwndDelay) {
				nDelayPos = nPos;
			}
			else if (hwndScroll == hwndAttenuation) {
				nAttenuationPos = nPos;
			}
			else {
				nGainPos = nPos;
			}

			nAttenuation = nAttenuationPos;
			if (nDelayPos > DELAYRANGE/2)
				nDelay = nDelayPos - DELAYRANGE/2; 
			else if (nDelayPos < DELAYRANGE/2)
				nDelay = nDelayPos + DELAYRANGE/2; 
			else nDelay = nDelayPos;
			nGain = nGainPos;
			
			break;
		default:
			break;
	} /* end switch (on message) */

    return (FALSE);
} //End fucntion (EchoParameters)
/****************************************************************************
    FUNCTION: ReverbParameters
    PURPOSE:  Processes messages for reverb parameters dialog box
****************************************************************************/
BOOL CALLBACK __export ReverbParameters(HWND hDlg,
                UINT message, WPARAM wParam, LPARAM lParam)
{
WORD nPos;
WORD wRange;
HWND hwndScroll;
static WORD nAttenuationPos = ATTENUATIONRANGE/2;
static WORD nAttenuationPos2 = ATTENUATIONRANGE/2;
static WORD nDelayPos = DELAYRANGE/2;
static WORD nDelayPos2 = DELAYRANGE/2;
static WORD nGainPos = GAINRANGE/2;
static HWND hwndDelay, hwndGain, hwndDelay2;
static HWND hwndAttenuation, hwndAttenuation2;

    switch (message) {
		case WM_INITDIALOG:                   //message: initialize dialog box
			hwndDelay = GetDlgItem (hDlg, IDC_SCROLLBAR1);
			hwndDelay2 = GetDlgItem (hDlg, IDC_SCROLLBAR2);
			hwndGain = GetDlgItem (hDlg, IDC_SCROLLBAR3);
			hwndAttenuation = GetDlgItem (hDlg, IDC_SCROLLBAR4);
			hwndAttenuation2 = GetDlgItem (hDlg, IDC_SCROLLBAR5);
			SetScrollRange (hwndDelay, SB_CTL, 0, DELAYRANGE, FALSE);
			SetScrollPos (hwndDelay, SB_CTL, nDelayPos, FALSE);
			SetScrollRange (hwndDelay2, SB_CTL, 0, DELAYRANGE, FALSE);
			SetScrollPos (hwndDelay2, SB_CTL, nDelayPos2, FALSE);
			SetScrollRange (hwndAttenuation, SB_CTL, 0, ATTENUATIONRANGE, FALSE);
			SetScrollPos (hwndAttenuation, SB_CTL, nAttenuationPos, FALSE);
			SetScrollRange (hwndAttenuation2, SB_CTL, 0, ATTENUATIONRANGE, FALSE);
			SetScrollPos (hwndAttenuation2, SB_CTL, nAttenuationPos2, FALSE);
			SetScrollRange (hwndGain, SB_CTL, 0, GAINRANGE, FALSE);
			SetScrollPos (hwndGain, SB_CTL, nGainPos, FALSE);
		    return (TRUE);
		case WM_COMMAND:                      //message: received a command
		    if (wParam == IDOK) {                //"OK" box selected?
				EndDialog(hDlg, TRUE);        //Exits the dialog box
				return (TRUE);
		    } //End if (OK button)
		    else if (wParam == IDCANCEL) {                //"CANCEL" box selected?
				EndDialog(hDlg, FALSE);        //Exits the dialog box
				return (TRUE);
		    } //End if (OK button)
		    break;
		case WM_VSCROLL:
			hwndScroll = (HWND) HIWORD (lParam);
			if (hwndScroll == hwndDelay) {
				wRange = DELAYRANGE;
				nPos = nDelayPos;
			}
			else if (hwndScroll == hwndDelay2) {
				wRange = DELAYRANGE;
				nPos = nDelayPos2;
			}
			else if (hwndScroll == hwndAttenuation) {
				wRange = ATTENUATIONRANGE;
				nPos = nAttenuationPos;
			}
			else if (hwndScroll == hwndAttenuation2) {
				wRange = ATTENUATIONRANGE;
				nPos = nAttenuationPos2;
			}
			else {
				wRange = GAINRANGE;
				nPos = nGainPos;
			}
			switch (wParam) {
				case SB_PAGEDOWN:
						nPos += wRange/10;
				case SB_LINEDOWN:
						nPos = min (wRange, nPos+10);
						break;
				case SB_PAGEUP:
						nPos -= wRange/10;
				case SB_LINEUP:
						nPos = max (0, nPos-10);
						break;
				case SB_TOP:
						nPos = 0;
						break;
				case SB_BOTTOM:
						nPos = wRange;
						break;
				case SB_THUMBPOSITION:
				case SB_THUMBTRACK:
						nPos = LOWORD (lParam);
						break;
				default:
					break;
			} /* end switch (wParam) */

			SetScrollPos (hwndScroll, SB_CTL, nPos, TRUE);
			if (hwndScroll == hwndDelay) {
				nDelayPos = nPos;
			}
			else if (hwndScroll == hwndDelay2) {
				nDelayPos2 = nPos;
			}
			else if (hwndScroll == hwndAttenuation) {
				nAttenuationPos = nPos;
			}
			else if (hwndScroll == hwndAttenuation2) {
				nAttenuationPos2 = nPos;
			}
			else {
				nGainPos = nPos;
			}

			nAttenuation = nAttenuationPos;
			nAttenuation2 = nAttenuationPos2;
			if (nDelayPos > DELAYRANGE/2)
				nDelay = nDelayPos - DELAYRANGE/2; 
			else if (nDelayPos < DELAYRANGE/2)
				nDelay = nDelayPos + DELAYRANGE/2; 
			else nDelay = nDelayPos;
			if (nDelayPos2 > DELAYRANGE/2)
				nDelay2 = nDelayPos2 - DELAYRANGE/2; 
			else if (nDelayPos2 < DELAYRANGE/2)
				nDelay2 = nDelayPos2 + DELAYRANGE/2; 
			else nDelay2 = nDelayPos2;
			nGain = nGainPos;
			
			break;
		default:
			break;
	} /* end switch (on message) */

    return (FALSE);
} //End fucntion (ReverbParameters)

